@@ -8,10 +8,11 @@
 from configparser import RawConfigParser, NoSectionError, NoOptionError
 from http.cookiejar import Cookie
 from pathlib import Path
+from typing import Any, Union
+
 from requests.cookies import RequestsCookieJar
 from selenium import webdriver
 from selenium.webdriver.chrome.options import Options
-from typing import Any, Union
 
 
 class OptionsManager(object):
@@ -21,7 +22,7 @@ class OptionsManager(object):
         """初始化，读取配置文件，如没有设置临时文件夹，则设置并新建  \n
         :param path: ini文件的路径，默认读取模块文件夹下的
         """
-        self.ini_path = str(Path(__file__).parent / 'configs.ini') if path == 'default' or path is None else path
+        self.ini_path = r'.\configs.ini' if path == 'default' or path is None else path
         self._conf = RawConfigParser()
         self._conf.read(self.ini_path, encoding='utf-8')
 
@@ -30,9 +31,9 @@ class OptionsManager(object):
         self._session_options = None
 
         if 'tmp_path' not in self.paths or not self.get_value('paths', 'tmp_path'):
-            tmp_path = str((Path(__file__).parent / 'tmp').absolute())
-            Path(tmp_path).mkdir(parents=True, exist_ok=True)
-            self.set_item('paths', 'tmp_path', tmp_path)
+            tmp_path = Path(r'.\tmp').absolute()
+            tmp_path.mkdir(parents=True, exist_ok=True)
+            self.set_item('paths', 'tmp_path', str(tmp_path))
             self.save(self.ini_path)
 
     def __text__(self) -> str:
@@ -114,7 +115,7 @@ class OptionsManager(object):
         :return: 当前对象
         """
         if path == 'default':
-            path = (Path(__file__).parent / 'configs.ini').absolute()
+            path = Path(r'.\configs.ini').absolute()
         elif path is None:
             path = Path(self.ini_path).absolute()
         else:
@@ -148,7 +149,7 @@ class SessionOptions(object):
         self._max_redirects = None
 
         if read_file:
-            self.ini_path = ini_path or str(Path(__file__).parent / 'configs.ini')
+            self.ini_path = ini_path or r'.\configs.ini'
             om = OptionsManager(self.ini_path)
             options_dict = om.session_options
 
@@ -267,7 +268,7 @@ class SessionOptions(object):
         :param headers: 参数值
         :return: None
         """
-        self._headers = {key.lower(): headers[key] for key in headers}
+        self.set_headers(headers)
 
     @cookies.setter
     def cookies(self, cookies: Union[RequestsCookieJar, list, tuple, str, dict]) -> None:
@@ -357,6 +358,14 @@ class SessionOptions(object):
         """
         self._max_redirects = max_redirects
 
+    def set_headers(self, headers: dict):
+        """设置headers参数           \n
+        :param headers: 参数值
+        :return: 返回当前对象
+        """
+        self._headers = {key.lower(): headers[key] for key in headers}
+        return self
+
     def set_a_header(self, attr: str, value: str):
         """设置headers中一个项          \n
         :param attr: 设置名称
@@ -389,7 +398,7 @@ class SessionOptions(object):
         :return: 返回当前对象
         """
         if path == 'default':
-            path = (Path(__file__).parent / 'configs.ini').absolute()
+            path = Path(r'.\configs.ini').absolute()
         elif path is None:
             path = Path(self.ini_path).absolute()
         else:
@@ -400,7 +409,7 @@ class SessionOptions(object):
         if path.exists():
             om = OptionsManager(str(path))
         else:
-            om = OptionsManager(self.ini_path or str(Path(__file__).parent / 'configs.ini'))
+            om = OptionsManager(self.ini_path or r'.\configs.ini')
 
         options = _session_options_to_dict(self)
 
@@ -431,7 +440,7 @@ class DriverOptions(Options):
         self.ini_path = None
 
         if read_file:
-            self.ini_path = ini_path or str(Path(__file__).parent / 'configs.ini')
+            self.ini_path = ini_path or r'.\configs.ini'
             om = OptionsManager(self.ini_path)
             options_dict = om.chrome_options
 
@@ -456,7 +465,7 @@ class DriverOptions(Options):
         :return: 当前对象
         """
         if path == 'default':
-            path = (Path(__file__).parent / 'configs.ini').absolute()
+            path = Path(r'.\configs.ini').absolute()
         elif path is None:
             path = Path(self.ini_path).absolute()
         else:
@@ -467,7 +476,7 @@ class DriverOptions(Options):
         if path.exists():
             om = OptionsManager(str(path))
         else:
-            om = OptionsManager(self.ini_path or str(Path(__file__).parent / 'configs.ini'))
+            om = OptionsManager(self.ini_path or r'.\configs.ini')
 
         options = self.as_dict()
 
@@ -670,12 +679,15 @@ def _dict_to_chrome_options(options: dict) -> Options:
     return chrome_options
 
 
-def _chrome_options_to_dict(options: Union[dict, DriverOptions, Options, None]) -> Union[dict, None]:
+def _chrome_options_to_dict(options: Union[dict, DriverOptions, Options, None, bool]) -> Union[dict, None]:
     """把chrome配置对象转换为字典                             \n
     :param options: chrome配置对象，字典或DriverOptions对象
     :return: 配置字典
     """
-    if isinstance(options, (dict, type(None))):
+    if options in (False, None):
+        return DriverOptions(read_file=False).as_dict()
+
+    if isinstance(options, dict):
         return options
 
     re_dict = dict()
@@ -693,7 +705,10 @@ def _session_options_to_dict(options: Union[dict, SessionOptions, None]) -> Unio
     :param options: session配置对象或字典
     :return: 配置字典
     """
-    if isinstance(options, (dict, type(None))):
+    if options in (False, None):
+        return SessionOptions(read_file=False).as_dict()
+
+    if isinstance(options, dict):
         return options
 
     re_dict = dict()
@@ -709,7 +724,7 @@ def _session_options_to_dict(options: Union[dict, SessionOptions, None]) -> Unio
         if val is not None:
             re_dict[attr] = val
 
-    # cert属性默认值为None，未免无法区分是否被设置，故主动赋值
+    # cert属性默认值为None，为免无法区分是否被设置，故主动赋值
     re_dict['cert'] = options.__getattribute__('_cert')
     re_dict['auth'] = options.__getattribute__('_auth')
 
