@@ -7,15 +7,15 @@
 from os import popen
 from pathlib import Path
 from pprint import pprint
-from re import search as RE_SEARCH
+from re import search as RE_SEARCH, sub
 from typing import Union
 
 from selenium import webdriver
 
-from DrissionPage.config import OptionsManager, DriverOptions
-from DrissionPage.drission import Drission
-from DrissionPage.session_page import SessionPage
 from .common import unzip
+from .config import OptionsManager, DriverOptions
+from .drission import Drission
+from .session_page import SessionPage
 
 
 def show_settings(ini_path: str = None) -> None:
@@ -203,7 +203,7 @@ def get_match_driver(ini_path: Union[str, None] = 'default',
     :param check_version: 是否检查版本匹配
     :return: None
     """
-    save_path = save_path or str(Path(__file__).parent)
+    save_path = save_path or '.'
 
     chrome_path = chrome_path or _get_chrome_path(ini_path, show_msg)
     chrome_path = Path(chrome_path).absolute() if chrome_path else None
@@ -349,7 +349,7 @@ def _download_driver(version: str, save_path: str = None, show_msg: bool = True)
         remote_main = i.text.split('.')[0]
 
         try:
-            remote_num = int(i.text.replace('.', '').replace('/', ''))
+            remote_num = int(sub(r'[./]', '', i.text))
         except ValueError:
             continue
 
@@ -359,7 +359,7 @@ def _download_driver(version: str, save_path: str = None, show_msg: bool = True)
 
     if remote_ver:
         url = f'https://cdn.npm.taobao.org/dist/chromedriver/{remote_ver}chromedriver_win32.zip'
-        save_path = save_path or Path(__file__).parent
+        save_path = save_path or '.'
         result = page.download(url, save_path, file_exists='overwrite', show_msg=show_msg)
 
         if result[0]:
