@@ -11,6 +11,7 @@ from requests.cookies import RequestsCookieJar
 from selenium.webdriver.chrome.webdriver import WebDriver
 from selenium.webdriver.remote.webelement import WebElement
 
+from .base import BasePage
 from .config import DriverOptions, SessionOptions
 from .drission import Drission
 from .driver_element import DriverElement
@@ -19,14 +20,7 @@ from .session_element import SessionElement
 from .session_page import SessionPage
 
 
-class Null(object):
-    """避免IDE发出未调用超类初始化函数的警告，无实际作用"""
-
-    def __init__(self):
-        pass
-
-
-class MixPage(Null, SessionPage, DriverPage):
+class MixPage(SessionPage, DriverPage, BasePage):
     """MixPage整合了DriverPage和SessionPage，封装了对页面的操作，
     可在selenium（d模式）和requests（s模式）间无缝切换。
     切换的时候会自动同步cookies。
@@ -35,47 +29,38 @@ class MixPage(Null, SessionPage, DriverPage):
     """
 
     def __init__(self,
-                 drission: Union[Drission, str] = None,
                  mode: str = 'd',
+                 drission: Union[Drission, str] = None,
                  timeout: float = 10,
                  driver_options: Union[dict, DriverOptions] = None,
                  session_options: Union[dict, SessionOptions] = None):
         """初始化函数                                                                         \n
-        :param drission: Drission对象，传入's'或'd'可自动创建Drission对象
         :param mode: 'd' 或 's'，即driver模式和session模式
+        :param drission: Drission对象，不传入时会自动创建
+        :param timeout: 超时时间，d模式时为寻找元素时间，s模式时为连接时间
         :param driver_options: 浏览器设置，没有传入drission参数时会用这个设置新建Drission对象
         :param session_options: requests设置，没有传入drission参数时会用这个设置新建Drission对象
         """
-        super().__init__()
-        if drission in ('s', 'd', 'S', 'D'):
-            mode = drission.lower()
-            drission = None
-
+        super(DriverPage, self).__init__(timeout)  # BasePage的__init__()
+        self._mode = mode.lower()
+        self._driver, self._session = (None, True) if self._mode == 's' else (True, None)
         self._drission = drission or Drission(driver_options, session_options)
-        self._url = None
-        self._response = None
-        self.timeout = timeout
-        self._url_available = None
-        self._mode = mode
-
-        self.retry_times = 3
-        self.retry_interval = 2
-
-        if mode == 's':
-            self._driver = None
-            self._session = True
-        elif mode == 'd':
-            self._driver = True
-            self._session = None
-        else:
-            raise ValueError("Argument mode can only be 'd' or 's'.")
 
     def __call__(self,
                  loc_or_str: Union[Tuple[str, str], str, DriverElement, SessionElement, WebElement],
                  mode: str = 'single',
-                 timeout: float = None):
-        return self.ele(loc_or_str, mode, timeout)
+                 timeout: float = None) \
+            -> Union[DriverElement, SessionElement, str, List[DriverElement], List[SessionElement]]:
+        """在内部查找元素                                            \n
+        例：ele = page('@id=ele_id')                               \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :param mode: 'single' 或 'all'，对应查找一个或全部
+        :param timeout: 超时时间
+        :return: DriverElement对象
+        """
+        return super().__call__(loc_or_str, mode, timeout)
 
+    # -----------------共有属性和方法-------------------
     @property
     def url(self) -> Union[str, None]:
         """返回当前url"""
@@ -85,15 +70,110 @@ class MixPage(Null, SessionPage, DriverPage):
             return self._session_url
 
     @property
-    def _session_url(self) -> str:
-        """返回session保存的url"""
-        return self._response.url if self._response else None
+    def title(self) -> str:
+        """返回网页title"""
+        if self._mode == 's':
+            return super().title
+        elif self._mode == 'd':
+            return super(SessionPage, self).title
 
     @property
-    def mode(self) -> str:
-        """返回当前模式，'s'或'd' """
-        return self._mode
+    def html(self) -> str:
+        """返回页面html文本"""
+        if self._mode == 's':
+            return super().html
+        elif self._mode == 'd':
+            return super(SessionPage, self).html
 
+    @property
+    def json(self) -> dict:
+        """当返回内容是json格式时，返回对应的字典"""
+        if self._mode == 's':
+            return super().json
+        elif self._mode == 'd':
+            return super(SessionPage, self).json
+
+    def get(self,
+            url: str,
+            go_anyway=False,
+            show_errmsg: bool = False,
+            retry: int = None,
+            interval: float = None,
+            **kwargs) -> Union[bool, None]:
+        """跳转到一个url                                         \n
+        跳转前先同步cookies，跳转后判断目标url是否可用
+        :param url: 目标url
+        :param go_anyway: 若目标url与当前url一致，是否强制跳转
+        :param show_errmsg: 是否显示和抛出异常
+        :param retry: 重试次数
+        :param interval: 重试间隔（秒）
+        :param kwargs: 连接参数，s模式专用
+        :return: url是否可用
+        """
+        if self._mode == 'd':
+            return super(SessionPage, self).get(url, go_anyway, show_errmsg, retry, interval)
+        elif self._mode == 's':
+            return super().get(url, go_anyway, show_errmsg, retry, interval, **kwargs)
+
+    def ele(self,
+            loc_or_ele: Union[Tuple[str, str], str, DriverElement, SessionElement, WebElement],
+            mode: str = None,
+            timeout: float = None) \
+            -> Union[DriverElement, SessionElement, str, List[SessionElement], List[DriverElement]]:
+        """返回页面中符合条件的元素、属性或节点文本，默认返回第一个                                               \n
+        :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
+        :param mode: 'single' 或 'all‘，对应查找一个或全部
+        :param timeout: 查找元素超时时间，d模式专用
+        :return: 元素对象或属性、文本节点文本
+        """
+        if self._mode == 's':
+            return super().ele(loc_or_ele, mode=mode)
+        elif self._mode == 'd':
+            return super(SessionPage, self).ele(loc_or_ele, mode=mode, timeout=timeout)
+
+    def eles(self,
+             loc_or_str: Union[Tuple[str, str], str],
+             timeout: float = None) -> Union[List[DriverElement], List[SessionElement]]:
+        """返回页面中所有符合条件的元素、属性或节点文本                                                           \n
+        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
+        :param timeout: 查找元素超时时间，d模式专用
+        :return: 元素对象或属性、文本节点文本组成的列表
+        """
+        return super(SessionPage, self).eles(loc_or_str, timeout=timeout)
+
+    def get_cookies(self, as_dict: bool = False, all_domains: bool = False) -> Union[dict, list]:
+        """返回cookies                               \n
+        :param as_dict: 是否以字典方式返回
+        :param all_domains: 是否返回所有域的cookies
+        :return: cookies信息
+        """
+        if self._mode == 's':
+            return super().get_cookies(as_dict, all_domains)
+        elif self._mode == 'd':
+            return super(SessionPage, self).get_cookies(as_dict)
+
+    def _try_to_connect(self,
+                        to_url: str,
+                        times: int = 0,
+                        interval: float = 1,
+                        mode: str = 'get',
+                        data: dict = None,
+                        show_errmsg: bool = False,
+                        **kwargs):
+        """尝试连接，重试若干次                            \n
+        :param to_url: 要访问的url
+        :param times: 重试次数
+        :param interval: 重试间隔（秒）
+        :param show_errmsg: 是否抛出异常
+        :param kwargs: 连接参数
+        :return: s模式为Response对象，d模式为bool
+        """
+        if self._mode == 'd':
+            return super(SessionPage, self)._try_to_connect(to_url, times, interval, show_errmsg)
+        elif self._mode == 's':
+            return super()._try_to_connect(to_url, times, interval, mode, data, show_errmsg, **kwargs)
+
+    # ----------------MixPage独有属性和方法-----------------------
     @property
     def drission(self) -> Drission:
         """返回当前使用的Dirssion对象"""
@@ -120,53 +200,14 @@ class MixPage(Null, SessionPage, DriverPage):
         return self._response
 
     @property
-    def cookies(self) -> Union[dict, list]:
-        """返回cookies"""
-        if self._mode == 's':
-            return super().cookies
-        elif self._mode == 'd':
-            return super(SessionPage, self).cookies
+    def mode(self) -> str:
+        """返回当前模式，'s'或'd' """
+        return self._mode
 
     @property
-    def html(self) -> str:
-        """返回页面html文本"""
-        if self._mode == 's':
-            return super().html
-        elif self._mode == 'd':
-            return super(SessionPage, self).html
-
-    @property
-    def title(self) -> str:
-        """返回网页title"""
-        if self._mode == 's':
-            return super().title
-        elif self._mode == 'd':
-            return super(SessionPage, self).title
-
-    def set_cookies(self, cookies: Union[RequestsCookieJar, list, tuple, str, dict], refresh: bool = True) -> None:
-        """设置cookies                                                          \n
-        :param cookies: cookies信息，可为CookieJar, list, tuple, str, dict
-        :param refresh: 设置cookies后是否刷新页面
-        :return: None
-        """
-        if self._mode == 's':
-            self.drission.set_cookies(cookies, set_session=True)
-        elif self._mode == 'd':
-            self.drission.set_cookies(cookies, set_driver=True)
-
-            if refresh:
-                self.refresh()
-
-    def get_cookies(self, as_dict: bool = False, all_domains: bool = False) -> Union[dict, list]:
-        """返回cookies                               \n
-        :param as_dict: 是否以字典方式返回
-        :param all_domains: 是否返回所有域的cookies
-        :return: cookies信息
-        """
-        if self._mode == 's':
-            return super().get_cookies(as_dict, all_domains)
-        elif self._mode == 'd':
-            return super(SessionPage, self).get_cookies(as_dict)
+    def _session_url(self) -> str:
+        """返回session保存的url"""
+        return self._response.url if self._response else None
 
     def change_mode(self, mode: str = None, go: bool = True) -> None:
         """切换模式，接收's'或'd'，除此以外的字符串会切换为d模式   \n
@@ -202,6 +243,19 @@ class MixPage(Null, SessionPage, DriverPage):
                 if go and self._drission.driver.current_url.startswith('http'):
                     self.get(self._drission.driver.current_url)
 
+    def set_cookies(self, cookies: Union[RequestsCookieJar, list, tuple, str, dict], refresh: bool = True) -> None:
+        """设置cookies                                                          \n
+        :param cookies: cookies信息，可为CookieJar, list, tuple, str, dict
+        :param refresh: 设置cookies后是否刷新页面
+        :return: None
+        """
+        if self._mode == 's':
+            self.drission.set_cookies(cookies, set_session=True)
+        elif self._mode == 'd':
+            self.drission.set_cookies(cookies, set_driver=True)
+            if refresh:
+                self.refresh()
+
     def cookies_to_session(self, copy_user_agent: bool = False) -> None:
         """从driver复制cookies到session                  \n
         :param copy_user_agent : 是否复制user agent信息
@@ -232,8 +286,18 @@ class MixPage(Null, SessionPage, DriverPage):
             r = self._make_response(self.url, **{'timeout': 3})[0]
             return r.ok if r else False
 
-    # ----------------重写SessionPage的函数-----------------------
+    def close_driver(self) -> None:
+        """关闭driver及浏览器"""
+        self._driver = None
+        self.drission.close_driver()
 
+    def close_session(self) -> None:
+        """关闭session"""
+        self._session = None
+        self._response = None
+        self.drission.close_session()
+
+    # ----------------重写SessionPage的函数-----------------------
     def post(self,
              url: str,
              data: dict = None,
@@ -287,7 +351,6 @@ class MixPage(Null, SessionPage, DriverPage):
                                 interval, **kwargs)
 
     # ----------------重写DriverPage的函数-----------------------
-
     def chrome_downloading(self, download_path: str = None) -> list:
         """返回浏览器下载中的文件列表                             \n
         :param download_path: 下载文件夹路径，默认读取配置信息
@@ -300,150 +363,5 @@ class MixPage(Null, SessionPage, DriverPage):
                 raise
         except:
             raise IOError('Download path not found.')
+
         return super().chrome_downloading(path)
-
-    # ----------------以下为共用函数-----------------------
-    def _try_to_connect(self,
-                        to_url: str,
-                        times: int = 0,
-                        interval: float = 1,
-                        mode: str = 'get',
-                        data: dict = None,
-                        show_errmsg: bool = False,
-                        **kwargs):
-        """尝试连接，重试若干次                            \n
-        :param to_url: 要访问的url
-        :param times: 重试次数
-        :param interval: 重试间隔（秒）
-        :param show_errmsg: 是否抛出异常
-        :param kwargs: 连接参数
-        :return: s模式为Response对象，d模式为bool
-        """
-        if self._mode == 'd':
-            return super(SessionPage, self)._try_to_connect(to_url, times, interval, show_errmsg)
-        elif self._mode == 's':
-            return super()._try_to_connect(to_url, times, interval, mode, data, show_errmsg, **kwargs)
-
-    def get(self,
-            url: str,
-            go_anyway=False,
-            show_errmsg: bool = False,
-            retry: int = None,
-            interval: float = None,
-            **kwargs) -> Union[bool, None]:
-        """跳转到一个url                                         \n
-        跳转前先同步cookies，跳转后判断目标url是否可用
-        :param url: 目标url
-        :param go_anyway: 若目标url与当前url一致，是否强制跳转
-        :param show_errmsg: 是否显示和抛出异常
-        :param retry: 重试次数
-        :param interval: 重试间隔（秒）
-        :param kwargs: 连接参数，s模式专用
-        :return: url是否可用
-        """
-        if self._mode == 'd':
-            return super(SessionPage, self).get(url, go_anyway, show_errmsg, retry, interval)
-        elif self._mode == 's':
-            return super().get(url, go_anyway, show_errmsg, retry, interval, **kwargs)
-
-    def ele(self,
-            loc_or_ele: Union[Tuple[str, str], str, DriverElement, SessionElement, WebElement],
-            mode: str = None,
-            timeout: float = None) \
-            -> Union[DriverElement, SessionElement, str, List[SessionElement], List[DriverElement]]:
-        """返回页面中符合条件的元素、属性或节点文本，默认返回第一个                                               \n
-        示例：                                                                                             \n
-        - 接收到元素对象时：                                                                                 \n
-            返回元素对象对象                                                                                 \n
-        - 用loc元组查找：                                                                                    \n
-            ele.ele((By.CLASS_NAME, 'ele_class'))        - 返回第一个class为ele_class的子元素                 \n
-        - 用查询字符串查找：                                                                                  \n
-            查找方式：属性、tag name和属性、文本、xpath、css selector、id、class                                \n
-            @表示属性，.表示class，#表示id，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串              \n
-            page.ele('.ele_class')                       - 返回第一个 class 为 ele_class 的元素               \n
-            page.ele('.:ele_class')                      - 返回第一个 class 中含有 ele_class 的元素            \n
-            page.ele('#ele_id')                          - 返回第一个 id 为 ele_id 的元素                     \n
-            page.ele('#:ele_id')                         - 返回第一个 id 中含有 ele_id 的元素                  \n
-            page.ele('@class:ele_class')                 - 返回第一个class含有ele_class的元素                  \n
-            page.ele('@name=ele_name')                   - 返回第一个name等于ele_name的元素                    \n
-            page.ele('@placeholder')                     - 返回第一个带placeholder属性的元素                   \n
-            page.ele('tag:p')                            - 返回第一个<p>元素                                  \n
-            page.ele('tag:div@class:ele_class')          - 返回第一个class含有ele_class的div元素               \n
-            page.ele('tag:div@class=ele_class')          - 返回第一个class等于ele_class的div元素               \n
-            page.ele('tag:div@text():some_text')         - 返回第一个文本含有some_text的div元素                 \n
-            page.ele('tag:div@text()=some_text')         - 返回第一个文本等于some_text的div元素                 \n
-            page.ele('text:some_text')                   - 返回第一个文本含有some_text的元素                    \n
-            page.ele('some_text')                        - 返回第一个文本含有some_text的元素（等价于上一行）      \n
-            page.ele('text=some_text')                   - 返回第一个文本等于some_text的元素                    \n
-            page.ele('xpath://div[@class="ele_class"]')  - 返回第一个符合xpath的元素                           \n
-            page.ele('css:div.ele_class')                - 返回第一个符合css selector的元素                    \n
-        - 查询字符串还有最精简模式，用x代替xpath、c代替css、t代替tag、tx代替text：                                  \n
-            page.ele('x://div[@class="ele_class"]')      - 等同于 page.ele('xpath://div[@class="ele_class"]') \n
-            page.ele('c:div.ele_class')                  - 等同于 page.ele('css:div.ele_class')               \n
-            page.ele('t:div')                            - 等同于 page.ele('tag:div')                         \n
-            page.ele('t:div@tx()=some_text')             - 等同于 page.ele('tag:div@text()=some_text')        \n
-            page.ele('tx:some_text')                     - 等同于 page.ele('text:some_text')                  \n
-            page.ele('tx=some_text')                     - 等同于 page.ele('text=some_text')
-        :param loc_or_ele: 元素的定位信息，可以是元素对象，loc元组，或查询字符串
-        :param mode: 'single' 或 'all‘，对应查找一个或全部
-        :param timeout: 查找元素超时时间，d模式专用
-        :return: 元素对象或属性、文本节点文本
-        """
-        if self._mode == 's':
-            return super().ele(loc_or_ele, mode=mode)
-        elif self._mode == 'd':
-            return super(SessionPage, self).ele(loc_or_ele, mode=mode, timeout=timeout)
-
-    def eles(self,
-             loc_or_str: Union[Tuple[str, str], str],
-             timeout: float = None) -> Union[List[DriverElement], List[SessionElement]]:
-        """返回页面中所有符合条件的元素、属性或节点文本                                                           \n
-        示例：                                                                                                \n
-        - 用loc元组查找：                                                                                      \n
-            page.eles((By.CLASS_NAME, 'ele_class'))       - 返回所有class为ele_class的元素                      \n
-        - 用查询字符串查找：                                                                                    \n
-            查找方式：属性、tag name和属性、文本、xpath、css selector、id、class                                  \n
-            @表示属性，.表示class，#表示id，=表示精确匹配，:表示模糊匹配，无控制字符串时默认搜索该字符串                \n
-            page.eles('.ele_class')                       - 返回所有 class 为 ele_class 的元素                  \n
-            page.eles('.:ele_class')                      - 返回所有 class 中含有 ele_class 的元素              \n
-            page.eles('#ele_id')                          - 返回所有 id 为 ele_id 的元素                        \n
-            page.eles('#:ele_id')                         - 返回所有 id 中含有 ele_id 的元素                    \n
-            page.eles('@class:ele_class')                 - 返回所有class含有ele_class的元素                    \n
-            page.eles('@name=ele_name')                   - 返回所有name等于ele_name的元素                      \n
-            page.eles('@placeholder')                     - 返回所有带placeholder属性的元素                     \n
-            page.eles('tag:p')                            - 返回所有<p>元素                                    \n
-            page.eles('tag:div@class:ele_class')          - 返回所有class含有ele_class的div元素                 \n
-            page.eles('tag:div@class=ele_class')          - 返回所有class等于ele_class的div元素                 \n
-            page.eles('tag:div@text():some_text')         - 返回所有文本含有some_text的div元素                   \n
-            page.eles('tag:div@text()=some_text')         - 返回所有文本等于some_text的div元素                   \n
-            page.eles('text:some_text')                   - 返回所有文本含有some_text的元素                      \n
-            page.eles('some_text')                        - 返回所有文本含有some_text的元素（等价于上一行）        \n
-            page.eles('text=some_text')                   - 返回所有文本等于some_text的元素                      \n
-            page.eles('xpath://div[@class="ele_class"]')  - 返回所有符合xpath的元素                              \n
-            page.eles('css:div.ele_class')                - 返回所有符合css selector的元素                       \n
-        - 查询字符串还有最精简模式，用x代替xpath、c代替css、t代替tag、tx代替text：                                    \n
-            page.eles('x://div[@class="ele_class"]')      - 等同于 page.eles('xpath://div[@class="ele_class"]') \n
-            page.eles('c:div.ele_class')                  - 等同于 page.eles('css:div.ele_class')               \n
-            page.eles('t:div')                            - 等同于 page.eles('tag:div')                         \n
-            page.eles('t:div@tx()=some_text')             - 等同于 page.eles('tag:div@text()=some_text')        \n
-            page.eles('tx:some_text')                     - 等同于 page.eles('text:some_text')                  \n
-            page.eles('tx=some_text')                     - 等同于 page.eles('text=some_text')
-        :param loc_or_str: 元素的定位信息，可以是loc元组，或查询字符串
-        :param timeout: 查找元素超时时间，d模式专用
-        :return: 元素对象或属性、文本节点文本组成的列表
-        """
-        if self._mode == 's':
-            return super().eles(loc_or_str)
-        elif self._mode == 'd':
-            return super(SessionPage, self).eles(loc_or_str, timeout=timeout)
-
-    def close_driver(self) -> None:
-        """关闭driver及浏览器"""
-        self._driver = None
-        self.drission.close_driver()
-
-    def close_session(self) -> None:
-        """关闭session"""
-        self._session = None
-        self._response = None
-        self.drission.close_session()
